package com.snowchen.snglfx17;

import cn.hutool.system.oshi.OshiUtil;
import com.snowchen.snglfx17.AppCore.ConfigCore;
import com.snowchen.snglfx17.AppCore.DownloadCore;
import com.snowchen.snglfx17.AppCore.LaunchCore;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.*;
import org.to2mbn.jmccc.launch.LaunchException;
import oshi.hardware.GlobalMemory;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;

public class HelloController {

    private String logFilePath;

    /**
     * 初始化方法，用于设置控制台输出重定向、初始化Java环境和配置
     * 此方法依次重定向系统输出和错误流到自定义的ConsoleOutputStream，
     * 以将输出信息显示在图形界面的LogArea中同时保持控制台输出
     * 此外，它还负责初始化Java环境，检查根目录，初始化版本选择和最大内存设置
     * 最后，它创建必要的目录和配置文件，以确保应用程序可以正常运行
     *
     * @throws InterruptedException 如果在初始化过程中线程被中断
     */
    public void initialize() throws InterruptedException, IOException {
        // 初始化日志文件路径
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss");
        logFilePath = "SNGL/log/" + dateFormat.format(new Date()) + ".log";
        System.out.println("日志文件路径: " + logFilePath);

        // 将标准输出流重定向到TextArea和文件
        System.setOut(new ConsoleOutputStream(System.out, LogArea::appendText, logFilePath));
        // 将标准错误流重定向到TextArea和文件
        System.setErr(new ConsoleOutputStream(System.err, LogArea::appendText, logFilePath));

        // 打印应用程序启动的ASCII艺术
        System.out.println(" ____  _   _  ____ _     \n" +
                "/ ___|| \\ | |/ ___| |    \n" +
                "\\___ \\|  \\| | |  _| |    \n" +
                " ___) | |\\  | |_| | |___ \n" +
                "|____/|_| \\_|\\____|_____|");

        // 初始化Java环境
        InitializeJavaEnviroment();
        // 检查根目录是否存在
        CheckRootDirectory();
        // 初始化版本选择
        InitializeVersionChoice();
        // 初始化最大内存设置
        InitializeMaxMem();
        // 设置最大内存监听
        SetMaxMemListen();

        // 定义目录名和文件名
        String directoryName = "SNGL";
        String directoryPath = "SNGL";
        String fileName = "sngl.json";

        // 创建目录，如果不存在的话
        ConfigCore.CreateDirectory.createDirectoryIfNotExists(directoryName);
        directoryName = ".minecraft";
        ConfigCore.CreateDirectory.createDirectoryIfNotExists(directoryName);

        // 检查并创建配置文件，如果不存在的话
        if (ConfigCore.CreateConfig.checkAndCreateFile(directoryPath, fileName)) {
            System.out.println("文件 " + fileName + " 已创建在 " + directoryPath + " 目录下");
        } else {
            System.out.println("文件 " + fileName + " 已存在于 " + directoryPath + " 目录下");
        }
        // 定义SNGL目录下的log目录名
        String logDirectoryName = "log";
        String logDirectoryPath = directoryPath + File.separator + logDirectoryName;

        // 创建SNGL目录下的log目录，如果不存在的话
        ConfigCore.CreateDirectory.createDirectoryIfNotExists(logDirectoryPath);
    }

    private static class ConsoleOutputStream extends java.io.PrintStream {
        private java.io.PrintStream original;
        private java.util.function.Consumer<String> consumer;
        private java.io.FileWriter fileWriter;

        public ConsoleOutputStream(java.io.PrintStream original, java.util.function.Consumer<String> consumer, String filePath) throws IOException {
            super(original);
            this.original = original;
            this.consumer = consumer;
            this.fileWriter = new java.io.FileWriter(filePath, true); // 追加模式
        }

        @Override
        public void println(final String x) {
            original.println(x); // 保持原始输出功能
            Platform.runLater(() -> consumer.accept(x + "\n")); // 在JavaFX应用程序线程上更新TextArea
            try {
                fileWriter.append(x + "\n"); // 写入文件
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            try {
                fileWriter.flush(); // 刷新缓冲区，确保写入
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void close() {
            super.close();
            try {
                fileWriter.close(); // 关闭文件写入流
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 显示错误提示对话框
     * 此方法用于在图形界面上向用户显示错误信息它创建一个弹出对话框，
     * 设置错误类型、标题和内容，并等待用户响应
     *
     * @param message 要显示的错误信息
     */
    private void showAlert(String message) {
        // 创建一个信息类型的警告对话框
        Alert alert = new Alert(Alert.AlertType.INFORMATION);
        // 设置对话框的标题为"错误"
        alert.setTitle("错误");
        // 设置对话框的头部文本为null，以简化界面
        alert.setHeaderText(null);
        // 设置对话框的内容文本为传入的错误信息
        alert.setContentText(message);
        // 显示对话框并等待用户响应
        alert.showAndWait();
    }

    @FXML // 启动按钮实现
/**
 * 当用户点击启动按钮时调用此方法
 * 它收集用户输入，如玩家名称、游戏版本、Java环境设置等，并启动游戏
 */
    protected void StartButtonClick() {
        // 获取玩家名称输入框中的文本
        String playerName = PlayerNameInput.getText().trim();
        if (playerName.isEmpty()) {
            showAlert("用户名不能为空");
            return;
        }
        LaunchCore.Player_Name = playerName;

        // 获取最大内存设置值，并进行范围检查
        double maxMemValue = SetMaxMem.getValue();
        if (maxMemValue <= 0 || maxMemValue > Integer.MAX_VALUE) {
            showAlert("最大内存设置值无效");
            return;
        }
        int maxMem = (int) maxMemValue;
        LaunchCore.Max_Mem = maxMem;

        // 获取版本选择框中选定的游戏版本
        String gameVersion = VersionChoice.getSelectionModel().getSelectedItem();
        if (gameVersion == null || gameVersion.isEmpty()) {
            showAlert("游戏版本不能为空");
            return;
        }
        LaunchCore.Game_Version = gameVersion;

        // 获取Java环境设置文本框中的文本，并进行验证
        String javaEnvironment = SetJavaEnviroment.getText().trim();
        if (javaEnvironment.isEmpty()) {
            showAlert("Java环境设置不能为空");
            return;
        }
        LaunchCore.Java_Environment = javaEnvironment;

        // 获取全屏设置复选框的状态
        boolean isFullScreen = SetFullScreen.isSelected();
        LaunchCore.FullScreen_Set = isFullScreen;

        // 获取游戏目录设置文本框中的文本，并进行验证
        String gameDirectory = SetGameDirectory.getText().trim();
        if (gameDirectory.isEmpty()) {
            showAlert("游戏目录不能为空");
            return;
        }
        if (!Objects.equals(gameDirectory, LaunchCore.Game_Directory)) {
            LaunchCore.Game_Directory = gameDirectory;
        }

        // 打印所有配置信息
        System.out.println("玩家名称: " + LaunchCore.Player_Name);
        System.out.println("最大内存设置: " + LaunchCore.Max_Mem);
        System.out.println("选择的游戏版本: " + LaunchCore.Game_Version);
        System.out.println("Java环境设置: " + LaunchCore.Java_Environment);
        System.out.println("全屏设置: " + LaunchCore.FullScreen_Set);
        System.out.println("游戏目录: " + LaunchCore.Game_Directory);

        // 尝试创建LaunchCore的新实例来启动游戏
        try {
            new LaunchCore();
        } catch (IOException | LaunchException e) {
            // 如果出现IOException或LaunchException，记录详细错误信息
            e.printStackTrace();
            throw new RuntimeException("启动游戏失败", e);
        }
    }


    /**
     * 检查并确保根目录下存在 .minecraft 目录及其子目录 versions
     * 如果这些目录不存在，将自动创建它们
     */
    private void CheckRootDirectory(){
        // 获取程序的根目录
        String rootPath = System.getProperty("user.dir");

        // 创建 .minecraft 目录的路径
        String minecraftDirectoryPath = rootPath + File.separator + ".minecraft";
        File minecraftDirectory = new File(minecraftDirectoryPath);

        // 如果 .minecraft 目录不存在，就创建它
        if (!minecraftDirectory.exists()) {
            minecraftDirectory.mkdir();
        }

        // 创建 versions 子目录的路径
        String versionsDirectoryPath = minecraftDirectoryPath + File.separator + "versions";
        File versionsDirectory = new File(versionsDirectoryPath);

        // 如果 versions 子目录不存在，就创建它
        if (!versionsDirectory.exists()) {
            versionsDirectory.mkdir();
        }
    }

    @FXML//刷新游戏文件夹按钮
    protected void ReloadGameDirectoryClick(){
        InitializeVersionChoice();
        System.out.println("游戏文件夹已重载");
    }

    private String[] VersionList = {};
    private void InitializeVersionChoice() {
        // 指定目录路径
        String directoryPath = SetGameDirectory.getText()+"/versions/";
        LaunchCore.Game_Directory = SetGameDirectory.getText();

        System.out.println(directoryPath);
        // 创建File对象，表示目录
        File directory = new File(directoryPath);

        // 确保指定的路径确实是一个目录
        if (!directory.isDirectory()) {
            System.out.println("指定的路径不是一个目录");
            showAlert("指定的路径不是一个目录");
            return;
        }

        // 使用FilenameFilter过滤出目录（而不是文件）
        FilenameFilter directoryFilter = new FilenameFilter() {
            @Override
            public boolean accept(File dir, String name) {
                // 接收目录，排除文件
                return new File(dir, name).isDirectory();
            }
        };

        // 获取目录下的所有目录名
        String[] VersionList = directory.list(directoryFilter);

        // 输出获取到的目录名
        for (String directoryName : VersionList) {
            System.out.println(directoryName);
        }
        ObservableList<String> optionList = FXCollections.observableArrayList(VersionList);
        VersionChoice.setItems(optionList);
        VersionChoice.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue != null) {
                System.out.println("Selected: " + newValue);
            }
        });
    }

    private void InitializeMaxMem(){//初始化最大内存设置
        GlobalMemory memory = OshiUtil.getMemory();
        double memoryTotal = memory.getTotal()/ (1024 * 1024);
        int intMemoryTotal = (int) memoryTotal;
        SetMaxMem.setMax(intMemoryTotal);
        SetMaxMem.setValue(intMemoryTotal/4);
        MaxMemInput.setText(String.valueOf(SetMaxMem.getValue()));
        LaunchCore.Max_Mem = (int) SetMaxMem.getValue();
    }

    @FXML
    private TextArea LogArea;//日志输出框

    @FXML
    private void onMaxMemTextChanged() {
        // 监听文本框的值变化，并实时更新滑块的最大值
        try {
            int maxMemValue = Integer.parseInt(MaxMemInput.getText());
            SetMaxMem.setValue(maxMemValue);
            LaunchCore.Max_Mem = maxMemValue;
        } catch (NumberFormatException e) {
            // 处理输入非数字的情况，可以选择给出提示或重置为默认值
            showAlert("请输入有效数字！");
        }
    }

    private void SetMaxMemListen() {//监听并更改滑块
        SetMaxMem.valueProperty().addListener((observable, oldValue, newValue) -> {
            MaxMemInput.setText(String.valueOf(newValue));
            LaunchCore.Max_Mem = (int) SetMaxMem.getValue();
        });
    }

    /**
     * 初始化Java环境方法
     * 该方法主要用于检测当前系统的Java版本和安装路径，
     * 并根据操作系统类型设置默认的Java执行文件路径
     */
    private void InitializeJavaEnviroment(){
        // 获取Java版本
        String javaVersion = System.getProperty("java.version");
        // 获取Java安装路径
        String javaHome = System.getProperty("java.home");
        // 设置Java版本信息显示文本
        JavaVersion.setText("Java版本:"+javaVersion);

        // 获取操作系统名称，并转换为小写以方便后续比较
        String osName = System.getProperty("os.name").toLowerCase();

        // 根据操作系统类型设置默认Java执行文件路径
        if (osName.contains("mac")) {
            // 对于MacOS系统，设置MacOS下的默认Java路径
            System.out.println("检测到系统为MacOS，设置默认Java路径");
            SetJavaEnviroment.setText(javaHome+"/bin/java");
            // 打印设置的Java路径以进行调试确认
            System.out.println(SetJavaEnviroment.getText());
        } else if (osName.contains("win")) {
            // 对于Windows系统，设置Windows下的默认Java路径
            System.out.println("检测到系统为Windows，设置默认Java路径");
            SetJavaEnviroment.setText(javaHome+"/bin/java.exe");
            // 打印设置的Java路径以进行调试确认
            System.out.println(SetJavaEnviroment.getText());
        } else {
            // 对于其他操作系统，简单打印提示信息
            System.out.println("其他操作系统");
        }
    }

    private void DataSave(){

    }

    @FXML
    protected void TestDownload(){
        System.out.println("开始下载1.20.1");
        DownloadCore.DownloadMinecraftVersion();
        //DownloadCore.DownloadRemoteVersionList();
        InitializeVersionChoice();
    }

    @FXML
    private CheckBox SetFullScreen;

    @FXML
    private TextField PlayerNameInput;//玩家名文本框
    @FXML
    private TextField MaxMemInput;//最大内存文本框

    @FXML
    private TextField ServerAddress;

    @FXML
    private TextField ServerPort;

    @FXML
    private TextField SetGameDirectory;

    @FXML
    private Button ReloadGameDirectory;

    @FXML
    private Slider SetMaxMem;

    @FXML
    private TextField SetJavaEnviroment;

    @FXML
    private ChoiceBox<String> VersionChoice;//版本选择下拉框

    @FXML
    private ChoiceBox<String> LoginModeChoice;//登录方式选择下拉框

    @FXML
    private ListView<String> MinecraftDownloadVersionList;

    @FXML
    private Label JavaVersion;

    @FXML
    private Button ManualSave;
}
